﻿using System;
using System.Collections;
using System.Diagnostics;
using System.Globalization;
using System.Windows.Data;

#pragma warning disable CA1714

namespace WpfApp1
{
    public class BindingDebugEnumerableConverter : IValueConverter  
    {
        [Flags]
        public enum BreakWhen
        {
            Null = 0x1,
            NotEnumerable=0x2,
            Empty=0x4,
            HasItems= 0x8,
            Default=Null | NotEnumerable | Empty | HasItems,
            NullOrEmpty = Null | Empty
        }

        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (BindingDebug.Convert)
            {
                var flags = parameter is BreakWhen bw ? bw : BreakWhen.Default;

                // break if value is null...
                if (value == null && flags.HasFlag(BreakWhen.Null))
                    Debugger.Break();

                // break if value is not enumerable
                if (value is IEnumerable enumerable)
                {
                    bool hasItems;

                    var enumerator = enumerable.GetEnumerator();

                    try
                    {
                        hasItems = enumerator.MoveNext();
                    }
                    finally
                    {
                        if (enumerator is IDisposable dsp)
                            dsp.Dispose();
                    }

                    if (flags.HasFlag(BreakWhen.Empty) && hasItems == false)
                        Debugger.Break();

                    if (flags.HasFlag(BreakWhen.HasItems) && hasItems)
                        Debugger.Break();
                }
            }

            return value;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return value;
        }
    }
}
