﻿using JetBrains.Annotations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace WpfApp1
{
    public class Data : INotifyPropertyChanged, IData
    {
        public void AddWorkspace(int workspaceId)
        {
            if (workspaceId <= 0)
                throw new ArgumentOutOfRangeException(nameof(workspaceId));

            mWorkspaces.Add(workspaceId);

            InvokePropertyChanged(nameof(Workspaces));
        }

        public void RemoveWorkspace(int workspaceId)
        {
            if (workspaceId <= 0)
                throw new ArgumentOutOfRangeException(nameof(workspaceId));

            mWorkspaces.Remove(workspaceId);

            InvokePropertyChanged(nameof(Workspaces));
        }

        public void ClearWorkspaces()
        {
            mWorkspaces.Clear();

            InvokePropertyChanged(nameof(Workspaces));
        }

        private readonly HashSet<int> mWorkspaces = new HashSet<int>(capacity: 1);

        public IReadOnlySet<int> Workspaces => mWorkspaces;

        #region File property

        private string mFile;

        public string File
        {
            get => mFile;
            set
            {
                if (mFile != value)
                {
                    mFile = value;

                    InvokePropertyChanged();
                }
            }
        }

        #endregion

        #region Speed property

        private double mSpeed;

        public double Speed
        {
            get => mSpeed;
            set
            {
                if (mSpeed != value)
                {
                    mSpeed = value;

                    InvokePropertyChanged();
                }
            }
        }

        #endregion

        #region RodForce property

        private double mRodForce;

        public double RodForce
        {
            get => mRodForce;
            set
            {
                if (mRodForce != value)
                {
                    mRodForce = value;

                    InvokePropertyChanged();
                }
            }
        }

        #endregion

        #region SealDrag property

        private double mSealDrag;

        public double SealDrag
        {
            get => mSealDrag;
            set
            {
                if (mSealDrag != value)
                {
                    mSealDrag = value;

                    InvokePropertyChanged();
                }
            }
        }

        #endregion

        #region IsComposite property

        private bool mIsComposite;

        public bool IsComposite
        {
            get => mIsComposite;
            set
            {
                if (mIsComposite != value)
                {
                    mIsComposite = value;

                    InvokePropertyChanged();
                }
            }
        }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void InvokePropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
