﻿using Bogus;

using System;
using System.Collections.Generic;
using System.Linq;

namespace WpfApp1
{
    public static class DataGenerator
    {
        private static readonly double[] smSpeeds =
            {
                1.0,
                2.0,
                3.0,
                5.0,
                10.0,
                12.0,
                15.0,
                20.0
            };

        private static readonly Faker<Data> smPVPGenerator = new Faker<Data>()
                                                             .FinishWith((f, data) => data.AddWorkspace(f.Random.Int(min: 1,max: 4)))
                                                             .RuleFor(o => o.File, f => f.Hacker.Phrase())
                                                             .RuleFor(o => o.IsComposite, f => true)
                                                             .RuleFor(o => o.RodForce, f => f.Random.Double(min: 20, max: 30))
                                                             .RuleFor(o => o.SealDrag, f => f.Random.Double(min: 2.0, max: 6.0))
                                                             .RuleFor(o => o.Speed, f => double.NaN);


        private static readonly Faker<Data> smCVPGenerator = new Faker<Data>()
                                                             .FinishWith((f, data) => data.AddWorkspace(f.Random.Int(min: 1, max: 4)))
                                                             .RuleFor(o => o.File, f => f.Hacker.Phrase())
                                                             .RuleFor(o => o.IsComposite, f => false)
                                                             .RuleFor(o => o.RodForce, f => f.Random.Double(min: 20, max: 30))
                                                             .RuleFor(o => o.SealDrag, f => f.Random.Double(min: 2.0, max: 6.0))
                                                             .RuleFor(o => o.Speed, f => f.PickRandom(smSpeeds));

        private static readonly Random smRandom = new Random(Seed: 123456);

        static DataGenerator()
        {
            Randomizer.Seed = smRandom;
        }

        private static IEnumerable<Data> GetPVP()
        {
            var res = new List<Data>();
            var composite = smPVPGenerator.Generate();
            var speeds = smRandom.Next(minValue: 3, smSpeeds.Length);

            res.Add(composite);

            for (var i = 0; i < speeds; ++i)
            {
                var spd = smCVPGenerator.Generate();
                spd.Speed = smSpeeds[i];
                spd.File = composite.File;

                // sett he workspace to match the parent speed
                spd.ClearWorkspaces();
                spd.AddWorkspace(composite.Workspaces.First());

                res.Add(spd);
            }

            return res;
        }

        public static IEnumerable<Data> GetData(int count)
        {
            if (count <= 0)
                throw new ArgumentOutOfRangeException(nameof(count));

            var res = new List<Data>();

            for (int i = 0; i < count; ++i)
            {
                switch (smRandom.Next(minValue: 0, maxValue: 2))
                {
                    case 0:
                        res.Add(smCVPGenerator.Generate());
                        break;

                    case 1:
                        res.AddRange(GetPVP());
                        break;
                }
            }

            return res;
        }
    }
}
