﻿using JetBrains.Annotations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace WpfApp1
{
    public class DataRow : IData, INotifyPropertyChanged
    {
        #region IsVelocityVisible property

        private bool mIsVelocityVisible;

        public bool IsVelocityVisible
        {
            get => mIsVelocityVisible;
            set
            {
                if (mIsVelocityVisible != value)
                {
                    mIsVelocityVisible = value;

                    InvokePropertyChanged();
                }
            }
        }

        #endregion

        #region IsDisplacementVisible property

        private bool mIsDisplacementVisible;

        public bool IsDisplacementVisible
        {
            get => mIsDisplacementVisible;
            set
            {
                if (mIsDisplacementVisible != value)
                {
                    mIsDisplacementVisible = value;

                    InvokePropertyChanged();
                }
            }
        }

        #endregion
		
        #region IsVisible property

        private bool mIsVisible = true;

        public bool IsVisible
        {
            get => mIsVisible;
            set
            {
                if (mIsVisible != value)
                {
                    mIsVisible = value;

                    InvokePropertyChanged();
                }
            }
        }

        #endregion

        #region RawIndex property

        private int mRawIndex;

        public int RawIndex
        {
            get => mRawIndex;
            set
            {
                if (mRawIndex != value)
                {
                    mRawIndex = value;

                    InvokePropertyChanged();
                }
            }
        }

        #endregion

        #region Item property

        private Data mItem;

        public Data Item
        {
            get => mItem;
            set
            {
                if (mItem != value)
                {
                    if (mItem != null)
                        mItem.PropertyChanged -= OnItemPropertyChanged;

                    mItem = value;

                    if (mItem != null)
                        mItem.PropertyChanged += OnItemPropertyChanged;

                    InvokePropertyChanged();
                }
            }
        }

        private void OnItemPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            InvokePropertyChanged(e?.PropertyName ?? string.Empty);
        }

        #endregion

        #region Children property

        private IEnumerable<DataRow> mChildren = Array.Empty<DataRow>();

        public IEnumerable<DataRow> Children
        {
            get => mChildren;
            set
            {
                if (mChildren != value)
                {
                    mChildren = value;

                    InvokePropertyChanged();
                }
            }
        }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void InvokePropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #region Implementation of IData

        public void AddWorkspace(int workspaceId)
        {
            mItem.AddWorkspace(workspaceId);
        }

        public void RemoveWorkspace(int workspaceId)
        {
            mItem.RemoveWorkspace(workspaceId);
        }

        public void ClearWorkspaces()
        {
            mItem.ClearWorkspaces();
        }

        IReadOnlySet<int> IData.Workspaces => mItem.Workspaces;

        public string File => mItem.File;

        public bool IsComposite => mItem.IsComposite;

        public double RodForce => mItem.RodForce;

        public double SealDrag => mItem.SealDrag;

        public double Speed => mItem.Speed;

        #endregion
    }
}
