﻿using JetBrains.Annotations;
using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;

#pragma warning disable CA1801, CA1823

namespace WpfApp1
{
    [PublicAPI]
    public sealed class ToggleSwitch : ToggleButton
    {
        #region Constructor

        static ToggleSwitch()
        {
            DefaultStyleKeyProperty.OverrideMetadata(smCtrlType, new FrameworkPropertyMetadata(smCtrlType));
        }

        #endregion

        #region Properties

        private const string CheckLabelAnimationName = "PART_CheckLabelAnimation";
        private const string SharedGroupStateName = "PART_SharedGroupSize";

        private const string HeaderPlacementVisualState = "HeaderContentPlacementAt";
        private const string HeaderStretchVisualState = "HeaderStretchAt";
        private const string SwitchPlacementVisualState = "SwitchContentPlacementAt";

        private const double DefaultSwitchWidthValue = 44.0D;
        private const string DefaultCheckedTextValue = "On";
        private const string DefaultUncheckedTextValue = "Off";
        private const Dock DefaultHeaderContentPlacementValue = Dock.Left;
        private const Dock DefaultSwitchContentPlacementValue = Dock.Left;
        private const HorizontalAlignment DefaultHeaderHorizontalValue = HorizontalAlignment.Left;
        private const HorizontalAlignment DefaultCheckHorizontalValue = HorizontalAlignment.Left;
        private const HorizontalAlignment DefaultSwitchHorizontalValue = HorizontalAlignment.Left;
        private static readonly string smDefaultSharedSizeGroupName = string.Empty;
        private static readonly Thickness smDefaultHeaderPaddingValue = new Thickness(uniformLength: 0D);
        private static readonly Thickness smDefaultCheckPaddingValue = new Thickness(uniformLength: 0D);
        private static readonly Thickness smDefaultSwitchPaddingValue = new Thickness(left: 8D, top: 0D, right: 8D, bottom: 0D);

        private static readonly Brush smCheckedBackgroundBrush = new SolidColorBrush(Color.FromRgb(r: 0x00, g: 0x63, b: 0xB1));
        private static readonly Brush smCheckedForegroundBrush = new SolidColorBrush(Color.FromRgb(r: 0xFF, g: 0xFF, b: 0xFF));
        private static readonly Brush smCheckedBorderBrushBrush = new SolidColorBrush(Color.FromRgb(r: 0x00, g: 0x63, b: 0xB1));
        private static readonly Brush smUncheckedBackgroundBrush = new SolidColorBrush(Color.FromRgb(r: 0xFF, g: 0xFF, b: 0xFF));
        private static readonly Brush smUncheckedForegroundBrush = new SolidColorBrush(Color.FromRgb(r: 0x33, g: 0x33, b: 0x33));
        private static readonly Brush smUncheckedBorderBrushBrush = new SolidColorBrush(Color.FromRgb(r: 0x33, g: 0x33, b: 0x33));

        private static readonly Type smCtrlType = typeof(ToggleSwitch);
        private const string CtrlName = nameof(ToggleSwitch);

        public static readonly DependencyProperty HeaderHorizontalAlignmentProperty =
            DependencyProperty.Register(nameof(HeaderHorizontalAlignment), typeof(HorizontalAlignment), smCtrlType, new PropertyMetadata(DefaultHeaderHorizontalValue, OnHeaderHorizontalAlignmentChanged));

        public static readonly DependencyProperty HeaderPaddingProperty =
            DependencyProperty.Register(nameof(HeaderPadding), typeof(Thickness), smCtrlType, new PropertyMetadata(smDefaultHeaderPaddingValue));

        public static readonly DependencyProperty HeaderContentPlacementProperty =
            DependencyProperty.Register(nameof(HeaderContentPlacement), typeof(Dock), smCtrlType, new PropertyMetadata(DefaultHeaderContentPlacementValue, OnHeaderContentPlacementPropertyChanged));

        public static readonly DependencyProperty CheckedTextProperty =
           DependencyProperty.Register(nameof(CheckedText), typeof(string), smCtrlType, new PropertyMetadata(DefaultCheckedTextValue, OnCheckTextChanged));

        public static readonly DependencyProperty UncheckedTextProperty =
            DependencyProperty.Register(nameof(UncheckedText), typeof(string), smCtrlType, new PropertyMetadata(DefaultUncheckedTextValue));

        public static readonly DependencyProperty CheckHorizontalAlignmentProperty =
            DependencyProperty.Register(nameof(CheckHorizontalAlignment), typeof(HorizontalAlignment), smCtrlType, new PropertyMetadata(DefaultCheckHorizontalValue));

        public static readonly DependencyProperty CheckPaddingProperty =
            DependencyProperty.Register(nameof(CheckPadding), typeof(Thickness), smCtrlType, new PropertyMetadata(smDefaultCheckPaddingValue));

        public static readonly DependencyProperty CheckedBackgroundProperty =
            DependencyProperty.Register(nameof(CheckedBackground), typeof(Brush), smCtrlType, new PropertyMetadata(smCheckedBackgroundBrush));

        public static readonly DependencyProperty CheckedForegroundProperty =
            DependencyProperty.Register(nameof(CheckedForeground), typeof(Brush), smCtrlType, new PropertyMetadata(smCheckedForegroundBrush));

        public static readonly DependencyProperty CheckedBorderBrushProperty =
            DependencyProperty.Register(nameof(CheckedBorderBrush), typeof(Brush), smCtrlType, new PropertyMetadata(smCheckedBorderBrushBrush));

        public static readonly DependencyProperty UncheckedBackgroundProperty =
            DependencyProperty.Register(nameof(UncheckedBackground), typeof(Brush), smCtrlType, new PropertyMetadata(smUncheckedBackgroundBrush));

        public static readonly DependencyProperty UncheckedForegroundProperty =
            DependencyProperty.Register(nameof(UncheckedForeground), typeof(Brush), smCtrlType, new PropertyMetadata(smUncheckedForegroundBrush));

        public static readonly DependencyProperty UncheckedBorderBrushProperty =
            DependencyProperty.Register(nameof(UncheckedBorderBrush), typeof(Brush), smCtrlType, new PropertyMetadata(smUncheckedBorderBrushBrush));

        public static readonly DependencyProperty SwitchWidthProperty =
            DependencyProperty.Register(nameof(SwitchWidth), typeof(Double), smCtrlType, new PropertyMetadata(DefaultSwitchWidthValue));

        public static readonly DependencyProperty SwitchPaddingProperty =
            DependencyProperty.Register(nameof(SwitchPadding), typeof(Thickness), smCtrlType, new PropertyMetadata(smDefaultSwitchPaddingValue));

        public static readonly DependencyProperty SwitchContentPlacementProperty =
            DependencyProperty.Register(nameof(SwitchContentPlacement), typeof(Dock), smCtrlType, new PropertyMetadata(DefaultSwitchContentPlacementValue, OnSwitchContentPlacementPropertyChanged));

        public static readonly DependencyProperty SwitchHorizontalAlignmentProperty =
            DependencyProperty.Register(nameof(SwitchHorizontalAlignment), typeof(HorizontalAlignment), smCtrlType, new PropertyMetadata(DefaultSwitchHorizontalValue));

        public static readonly DependencyProperty SharedSizeGroupNameProperty =
            DependencyProperty.Register(nameof(SharedSizeGroupName), typeof(string), smCtrlType, typeMetadata: null);

        [Browsable(browsable: false)]
        [EditorBrowsable(EditorBrowsableState.Never)]
        public static readonly DependencyProperty IsHeaderStretchProperty = DependencyProperty.Register(nameof(IsHeaderStretch), typeof(bool), smCtrlType, typeMetadata: null);

        [Bindable(bindable: true)]
        [Description("Gets or sets the horizontal alignment of the Header's content."), Category(CtrlName)]
        public HorizontalAlignment HeaderHorizontalAlignment
        {
            get => (HorizontalAlignment)GetValue(HeaderHorizontalAlignmentProperty);
            set => SetValue(HeaderHorizontalAlignmentProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the spacing around the header text."), Category(CtrlName)]
        public Thickness HeaderPadding
        {
            get => (Thickness)GetValue(HeaderPaddingProperty);
            set => SetValue(HeaderPaddingProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the placement of the Header Content to the Switch."), Category(CtrlName)]
        public Dock HeaderContentPlacement
        {
            get => (Dock)GetValue(HeaderContentPlacementProperty);
            set => SetValue(HeaderContentPlacementProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the Switch text for checked state."), Category(CtrlName)]
        public string CheckedText
        {
            get => (string)GetValue(CheckedTextProperty);
            set => SetValue(CheckedTextProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the Switch text for Unchecked state."), Category(CtrlName)]
        public string UncheckedText
        {
            get => (string)GetValue(UncheckedTextProperty);
            set => SetValue(UncheckedTextProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the horizontal alignment of the Check's content."), Category(CtrlName)]
        public HorizontalAlignment CheckHorizontalAlignment
        {
            get => (HorizontalAlignment)GetValue(CheckHorizontalAlignmentProperty);
            set => SetValue(CheckHorizontalAlignmentProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the spacing around the switch text."), Category(CtrlName)]
        public Thickness CheckPadding
        {
            get => (Thickness)GetValue(CheckPaddingProperty);
            set => SetValue(CheckPaddingProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the graphical switch checked background brush."), Category(CtrlName)]
        public Brush CheckedBackground
        {
            get => (Brush)GetValue(CheckedBackgroundProperty);
            set => SetValue(CheckedBackgroundProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the graphical switch checked foreground brush."), Category(CtrlName)]
        public Brush CheckedForeground
        {
            get => (Brush)GetValue(CheckedForegroundProperty);
            set => SetValue(CheckedForegroundProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the graphical switch checked border brush."), Category(CtrlName)]
        public Brush CheckedBorderBrush
        {
            get => (Brush)GetValue(CheckedBorderBrushProperty);
            set => SetValue(CheckedBorderBrushProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the graphical switch Unchecked background brush."), Category(CtrlName)]
        public Brush UncheckedBackground
        {
            get => (Brush)GetValue(UncheckedBackgroundProperty);
            set => SetValue(UncheckedBackgroundProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the graphical switch Unchecked foreground brush."), Category(CtrlName)]
        public Brush UncheckedForeground
        {
            get => (Brush)GetValue(UncheckedForegroundProperty);
            set => SetValue(UncheckedForegroundProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the graphical switch Unchecked border brush."), Category(CtrlName)]
        public Brush UncheckedBorderBrush
        {
            get => (Brush)GetValue(UncheckedBorderBrushProperty);
            set => SetValue(UncheckedBorderBrushProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the width of the graphical switch. (Default: 44)"), Category(CtrlName)]
        public Double SwitchWidth
        {
            get => (Double)GetValue(SwitchWidthProperty);
            set => SetValue(SwitchWidthProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the spacing around the graphical switch."), Category(CtrlName)]
        public Thickness SwitchPadding
        {
            get => (Thickness)GetValue(SwitchPaddingProperty);
            set => SetValue(SwitchPaddingProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the placement of the graphical switch to the state text."), Category(CtrlName)]
        public Dock SwitchContentPlacement
        {
            get => (Dock)GetValue(SwitchContentPlacementProperty);
            set => SetValue(SwitchContentPlacementProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Gets or sets the horizontal alignment of the Switch's content."), Category(CtrlName)]
        public HorizontalAlignment SwitchHorizontalAlignment
        {
            get => (HorizontalAlignment)GetValue(SwitchHorizontalAlignmentProperty);
            set => SetValue(SwitchHorizontalAlignmentProperty, value);
        }

        [Bindable(bindable: true)]
        [Description("Name of Shared Size Group for the Left column (header or switch). Depends on HeaderContentPlacement property."), Category(CtrlName)]
        public string SharedSizeGroupName
        {
            get => (string)GetValue(SharedSizeGroupNameProperty);
            set => SetValue(SharedSizeGroupNameProperty, value);
        }

        [Browsable(browsable: false)]
        [EditorBrowsable(EditorBrowsableState.Never)]
        public bool IsHeaderStretch
        {
            get => (bool)GetValue(IsHeaderStretchProperty);
            set => SetValue(IsHeaderStretchProperty, value);
        }

        #endregion

        #region Events

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            CheckLabelAnimationValue(this);
            SharedGroupStateValue(this, HeaderContentPlacement);
            CoerceHeaderSizing();
            UpdatePlacementVisualState();
        }

        private static void OnHeaderHorizontalAlignmentChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if (d.TryCast(out ToggleSwitch ctrl))
                ctrl.CoerceHeaderSizing();
        }

        private void CoerceHeaderSizing()
        {
            SetValue(IsHeaderStretchProperty, HeaderContentPlacement == Dock.Left || HeaderContentPlacement == Dock.Right);
        }

        private static void OnCheckTextChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if (d.TryCast(out ToggleSwitch ctrl))
                CheckLabelAnimationValue(ctrl);
        }

        private static void OnHeaderContentPlacementPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if (d.TryCast(out ToggleSwitch ctrl))
            {
                var oldValue = (Dock)e.OldValue;
                var newValue = (Dock)e.NewValue;

                ChangeSharedGroupStateValue(ctrl, newValue, oldValue);
                ctrl.OnHeaderContentPlacementChanged(newValue, oldValue);
            }
        }

        private static void OnSwitchContentPlacementPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if (d.TryCast(out ToggleSwitch ctrl))
                ctrl.OnSwitchContentPlacementChanged((Dock)e.NewValue, (Dock)e.OldValue);
        }

        #endregion

        #region Methods

        private static void ChangeSharedGroupStateValue(ToggleSwitch ctrl, Dock newValue, Dock oldValue)
        {
            SharedGroupStateValue(ctrl, oldValue, isBound: false);
            SharedGroupStateValue(ctrl, newValue);
        }

        private static void SharedGroupStateValue(ToggleSwitch ts, Dock placement, bool isBound = true)
        {
            var field = (DiscreteObjectKeyFrame)ts.Template?.FindName(SharedGroupStateName + placement, ts);
            if (field != null)
            {
                var binding = new Binding(nameof(SharedSizeGroupName)) { Source = ts };
                BindingOperations.SetBinding(field, ObjectKeyFrame.ValueProperty, isBound ? binding : new Binding());
            }
        }

        private static void CheckLabelAnimationValue(ToggleSwitch ts)
        {
            var checkLabelAnimation = (DiscreteObjectKeyFrame)ts.Template?.FindName(CheckLabelAnimationName, ts);
            if (checkLabelAnimation != null)
            {
                var binding = new Binding(nameof(CheckedText)) { Source = ts };
                BindingOperations.SetBinding(checkLabelAnimation, ObjectKeyFrame.ValueProperty, binding);
            }
        }

        private void OnHeaderContentPlacementChanged(Dock newValue, Dock oldValue)
        {
            CoerceHeaderSizing();
            UpdateHeaderVisualState(newValue);
        }

        private void OnSwitchContentPlacementChanged(Dock newValue, Dock oldValue)
        {
            UpdateSwitchPlacementVisualState(newValue);
        }

        private void UpdatePlacementVisualState()
        {
            UpdateHeaderVisualState(HeaderContentPlacement);
            UpdateSwitchPlacementVisualState(SwitchContentPlacement);
        }

        private void UpdateHeaderVisualState(Dock newPlacement)
        {
            GoToState($"{HeaderPlacementVisualState}{newPlacement.ToString()}", useTransitions: false);

            if (IsHeaderStretch)
            {
                switch (newPlacement)
                {
                    case Dock.Right:
                    case Dock.Left:
                        GoToState($"{HeaderStretchVisualState}{newPlacement.ToString()}", useTransitions: false);
                        break;
                    case Dock.Top:
                    case Dock.Bottom:
                        GoToState($"{HeaderStretchVisualState}Middle", useTransitions: false);
                        break;
                }
            }
            else
                GoToState($"{HeaderStretchVisualState}Middle", useTransitions: false);
        }

        private void UpdateSwitchPlacementVisualState(Dock newPlacement)
        {
            GoToState($"{SwitchPlacementVisualState}{newPlacement.ToString()}", useTransitions: false);
        }

        internal bool GoToState(string stateName, bool useTransitions)
        {
            return VisualStateManager.GoToState(this, stateName, useTransitions);
        }

        #endregion
    }
}